"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var EventRegistry_1 = __importDefault(require("./EventRegistry"));
var InMemoryCollection_1 = __importDefault(require("./InMemoryCollection"));
var maxSize = 100;
var ChannelMembershipsStorage = /** @class */ (function () {
    function ChannelMembershipsStorage() {
        this.channelMembershipsCollection = new InMemoryCollection_1.default('id');
        this.eventRegistry = new EventRegistry_1.default();
    }
    ChannelMembershipsStorage.prototype.upsert = function (channelMemberships) {
        var len = channelMemberships.length;
        for (var i = 0; i < len; i++) {
            // only upsert unempty channelMemberships
            this.channelMembershipsCollection.upsert(channelMemberships[i]);
        }
    };
    ChannelMembershipsStorage.prototype.update = function (channelMemberships) {
        if (this.channelMembershipsCollection.hasChanges(channelMemberships)) {
            this.upsert(channelMemberships);
            this.eventRegistry.emit('update', {});
        }
    };
    ChannelMembershipsStorage.prototype.sortedChannelMemberships = function () {
        return this.channelMembershipsCollection.sort('id', 'desc');
    };
    ChannelMembershipsStorage.prototype.findBy = function (filter) {
        return this.channelMembershipsCollection.findBy(filter);
    };
    ChannelMembershipsStorage.prototype.findMany = function (channelId) {
        var doc = this.channelMembershipsCollection.findMany({ channelId: channelId });
        return doc;
    };
    ChannelMembershipsStorage.prototype.state = function () {
        var channelMemberships = this.sortedChannelMemberships();
        return {
            channelMemberships: channelMemberships.filter(function (channelMembership) { return channelMembership.channelId; }),
        };
    };
    ChannelMembershipsStorage.prototype.onUpdate = function (callback) {
        return this.eventRegistry.on('update', callback);
    };
    return ChannelMembershipsStorage;
}());
exports.default = ChannelMembershipsStorage;
//# sourceMappingURL=ChannelMembershipStorage.js.map