import ChannelMessageStorage from './ChannelMessageStorage';
import ChannelsStorage from './ChannelStorage';
import { ChatMessage, ChatMessageType, ChatChannel, ChatUser, IActingType, ChannelMembership } from './ChatClient';
import ChannelMembershipsStorage from './ChannelMembershipStorage';
declare class ChatService {
    private chatMemoryStorageAdapter;
    private chatClient;
    private channelsStorage;
    private channelMembershipsStorage;
    private lastActingAt;
    private lastActingType;
    private currentActiveChannel;
    private static chatService;
    private static token;
    currentUser: ChatUser | null;
    static getService(token: string, endpoint?: string): ChatService;
    constructor(endpoint: string, token: string);
    start(): void;
    setCurrentActiveChannel(channel: ChatChannel | null): void;
    getCurrentActiveChannel(): ChatChannel | null;
    channelMessageStorage(channelId: string): ChannelMessageStorage;
    getChannelStorage(): ChannelsStorage;
    getChannelMembershipsStorage(): ChannelMembershipsStorage;
    getCurrentUser(): Promise<ChatUser>;
    getChannel(channelId: string): Promise<ChatChannel>;
    getChannelByExternalId(channelExternalId: string): Promise<ChatChannel>;
    subscribeChannel(channelId: string): CallableFunction;
    updateChannelMemberships(channelMemberships: ChannelMembership[]): void;
    subscribeChannelsUpdate(): () => void;
    sendMessage(channelId: string, messageType: ChatMessageType, content: string, metadata?: Record<string, unknown>, replyTo?: ChatMessage | null, localId?: string): Promise<void>;
    retryMessage(message: ChatMessage, onUploadProgress?: (progress: number) => any): Promise<any>;
    triggerUserTyping(isTyping: boolean, channelExternalId: string): Promise<void>;
    triggerUserRecordingVoice(isRecording: boolean, channelExternalId: string): Promise<void>;
    triggerUserRecordingVideo(isRecording: boolean, channelExternalId: string): Promise<void>;
    triggerUserActing(isActing: boolean, actingType: IActingType, channelExternalId: string): Promise<void>;
    markAllAsRead(channelId: string): void;
    sendAttachment(channelId: string, messageType: ChatMessageType, content: string, file: any, replyTo?: ChatMessage | null, localId?: string, onUploadProgress?: (progress: number) => any): Promise<void>;
    sendMessageExternally(channelExternalId: string, messageType: ChatMessageType, content: string, metadata?: Record<string, unknown>, replyToId?: string): Promise<void>;
    loadEarlier(channelId: string): Promise<void>;
    loadLater(channelId: string): Promise<void>;
    searchMessages(channelId: string, query: string): Promise<ChatMessage[]>;
    getUser(id: string): Promise<ChatUser>;
    blockUser(blockedId: string): Promise<void>;
    unBlockUser(blockedId: string): Promise<void>;
    archiveChannel(channelId: string): Promise<void>;
    unArchiveChannel(channelId: string): Promise<void>;
    draftMessage(channelId: string, draft: string): Promise<void>;
    deleteMessage(messageId: string): Promise<any>;
    loadEarlierChannels(): void;
    getChannels(filter?: {
        skipEmpty?: boolean;
        metadata?: Record<any, any>;
        perPage?: number;
    }): Promise<ChatChannel[]>;
    getAllUnreadMessageCount(): Promise<{
        unreadMessageCount: number;
    }>;
}
export default ChatService;
//# sourceMappingURL=ChatService.d.ts.map