import { ChannelMembershipStoragStateType } from './useChannelMembershipsData';
import { ChannelMembership } from './ChatClient';
import EventRegistry from './EventRegistry';
import InMemoryCollection from './InMemoryCollection';

const maxSize = 100;
class ChannelMembershipsStorage {
  private eventRegistry: EventRegistry;
  private channelMembershipsCollection: InMemoryCollection<ChannelMembership>;

  constructor() {
    this.channelMembershipsCollection = new InMemoryCollection('id');
    this.eventRegistry = new EventRegistry();
  }

  upsert(channelMemberships: ChannelMembership[]) {
    const len = channelMemberships.length;
    for (let i = 0; i < len; i++) {
      // only upsert unempty channelMemberships
      this.channelMembershipsCollection.upsert(channelMemberships[i]);
    }
  }

  update(channelMemberships: ChannelMembership[]) {
    if (this.channelMembershipsCollection.hasChanges(channelMemberships)) {
      this.upsert(channelMemberships);
      this.eventRegistry.emit('update', {});
    }
  }

  sortedChannelMemberships() {
    return this.channelMembershipsCollection.sort('id', 'desc');
  }

  findBy(filter: { channelId: string; userId: string }) {
    return this.channelMembershipsCollection.findBy(filter);
  }

  findMany(channelId: string): ChannelMembership[] | undefined {
    const doc = this.channelMembershipsCollection.findMany({ channelId });
    return doc;
  }

  state(): ChannelMembershipStoragStateType {
    const channelMemberships = this.sortedChannelMemberships();
    return {
      channelMemberships: channelMemberships.filter(
        (channelMembership) => channelMembership.channelId
      ),
    };
  }

  onUpdate(callback: (data: any) => void) {
    return this.eventRegistry.on('update', callback);
  }
}

export default ChannelMembershipsStorage;
