import {v4 as uuidv4} from 'uuid';
type Callback = (data: any) => void;
type Filter = (data: any) => boolean;
type FilterableCallback = {
  callback: Callback;
  filter?: Filter;
};
type RemoveCallBackHandle = () => void;

class EventRegistry {
  private callbacks: {[eventName: string]: {[id: string]: FilterableCallback}} = {};

  on(eventName: string, callback: Callback, filter?: Filter): RemoveCallBackHandle {
    this.callbacks[eventName] = this.callbacks[eventName] || {};
    const id = uuidv4();
    this.callbacks[eventName][id] = {callback, filter};

    return () => {
      delete this.callbacks[eventName][id];
    };
  }

  emit(eventName: string, data: any) {
    if (this.callbacks[eventName]) {
      Object.values(this.callbacks[eventName]).forEach((filterableCallback) => {
        if (filterableCallback.filter && filterableCallback.filter(data) === false) return;
        filterableCallback.callback(data);
      });
    }
  }
}

export default EventRegistry;
