import isEqual from 'lodash/isEqual';
import { isSubset } from './utils';

type ItemConstraint = Record<string, any>;

class InMemoryCollection<Item extends ItemConstraint> {
  private items: Record<string, Item> = {};
  private uniqueKey: keyof Item;
  constructor(uniqueKey: keyof Item) {
    this.uniqueKey = uniqueKey;
  }

  public compare(item1: Item, item2: Item) {
    return isEqual(item1, item2);
  }

  public hasChanges(collection1: Item[]) {
    for (let i = 0; i < collection1.length; i++) {
      const existingItem = this.items[collection1[i][this.uniqueKey]];
      if (!existingItem || !isSubset(existingItem, collection1[i])) {
        return true;
      }
    }
    return false;
  }

  public upsert(item: Item) {
    this.items[item[this.uniqueKey]] = {
      ...this.items[item[this.uniqueKey]],
      ...item,
    };
    return item;
  }
  public find(id: string) {
    return this.items[id];
  }

  public remove(id: string) {
    delete this.items[id];
  }

  public findBy(filter: any) {
    return Object.values(this.items).find((item) => {
      return Object.keys(filter).every(
        (filterKey) => filter[filterKey] === item[filterKey]
      );
    });
  }
  public findMany(filter: any) {
    return Object.values(this.items).filter((item) => {
      return Object.keys(filter).every(
        (filterKey) => filter[filterKey] === item[filterKey]
      );
    });
  }

  public sort(key: keyof Item, dir: 'asc' | 'desc' = 'asc') {
    return Object.values(this.items).sort((i1, i2) =>
      i1[key] > i2[key] ? (dir === 'asc' ? 1 : -1) : dir === 'asc' ? -1 : 1
    );
  }
}

export default InMemoryCollection;
