import { ChatMessage, ChatChannel, ChannelMembership } from './ChatClient';

export function serializeMessage(message: ChatMessage): ChatMessage {
  return {
    ...message,
    id: String(message.id),
    senderId: String(message.senderId),
    localId: message.localId ? String(message.localId) : message.localId,
    channelExternalId: String(message.channelExternalId),
    channelId: String(message.channelId),
    replyToId: String(message.replyToId),
  };
}

export function serializeMessages(messages: ChatMessage[]) {
  return messages.filter((message) => message).map(serializeMessage);
}

export function serializeChannelMembership(
  channelMembership: ChannelMembership
): ChannelMembership {
  return {
    ...channelMembership,
    id: String(channelMembership.id),
    lastReadMessageId: channelMembership.lastReadMessageId
      ? parseInt(String(channelMembership.lastReadMessageId), 10)
      : channelMembership.lastReadMessageId,
    lastRecievedMessageId: channelMembership.lastRecievedMessageId
      ? parseInt(String(channelMembership.lastRecievedMessageId), 10)
      : channelMembership.lastRecievedMessageId,
    userId: String(channelMembership.userId),
    channelId: String(channelMembership.channelId),
  };
}

export function serializeChannelMemberships(
  channelMemberships: ChannelMembership[]
) {
  return channelMemberships.map(serializeChannelMembership);
}

export function serializeChannel(channel: ChatChannel): ChatChannel {
  return {
    ...channel,
    id: String(channel.id),
    externalId: String(channel.externalId),
    lastMessage: channel.lastMessage
      ? serializeMessage(channel.lastMessage)
      : null,
    channelMemberships: serializeChannelMemberships(
      channel.channelMemberships || []
    ),
  };
}

export function serializeChannels(channels: ChatChannel[]) {
  return channels.map(serializeChannel);
}
